#!/bin/bash
#Copyright (C) 2019 Hewlett-Packard Development Company, L.P.
#
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License version 2 as
#published by the Free Software Foundation.
#


login=""
colors=1

display_usage() {
  echo -e "${white}\nThis script will check the firmware version"
  echo "of all the ssd in the system"
  echo "${bold}${green}usage:${normal} $(basename $0) [-hduph]"
  echo "  -h      display help"
  echo "  -u      iLO username"
  echo "  -p      iLO password"
  echo "  -H      iLO hostname"
  echo "  -n      Force no colors [Autodetect by default]"
  exit 1
}

IlorestLogout (){
  ./ilorest  --nologo logout
}

check_model () {
  local ProblemModels="EK0800JVYPN EO1600JVYPP MK0800JVYPQ MO1600JVYPR"
  local seeking=$1
  local in=0
  for element in ${ProblemModels}; do
    if [[ $element == "$seeking" ]]; then
      in=1
      break
    fi
  done
  echo $in
}


runilorest() {
  output=$(./ilorest  --nologo $1 ${login} 2>/dev/null)
  retcode=$?
  if [ ${retcode} -eq 23 ]; then 
    pr_info "Unable to find Smart Storage Drive information."
    IlorestLogout
    return 1
  elif [ ${retcode} -eq 34 ]; then
    pr_info "If running locally CHIF driver is required. If running remotely, --url, --user, and --password arguments are required."
    return 1
  elif [ ${retcode} -eq 32 ]; then
    pr_info "Error: Could not authenticate. Invalid credentials, or bad username/password."
    return 1
  elif [ ${retcode} -eq 75 ]; then
    pr_info "Local Higher Security Mode is detected, please include --username and --password with iLO credentials."
    return 1
  elif [ ${retcode} -eq 5 ]; then
    pr_info "If running locally you must run this script as an Administrator."
    return 1
  elif [ ${retcode} -ne 0 ]; then
    pr_info "A general error occurred."
    return 1
    IlorestLogout
  fi
  echo "${output}"
}

GetServerInfo() {
  pr_info "SSD Firmware $(date '+%m/%d/%Y %H:%M:%S' )"
  pr_info "=================================================================="
  pr_info "Server Info:"
  pr_info "=================================================================="
  hostname=$(runilorest "get Oem/Hpe/Manager/HostName --selector ServiceRoot." |grep -m1 HostName|cut -d= -f2)
  if [[ -z "${hostname}" ]]; then
    hostname=$(runilorest "get Oem/Hp/Manager/HostName --selector ServiceRoot." |grep -m1 HostName|cut -d= -f2)
  fi
  serverdata=$(runilorest "get ServerName SerialNumber --selector Bios.")
  pr_info "SerialNumber : ${normal}$(echo "${serverdata}"|grep SerialNumber|cut -d= -f2)"
  pr_info "ServerName   : ${normal}$(echo "${serverdata}"|grep ServerName|cut -d= -f2)"
  pr_info "iLO HostName : ${normal}${hostname}"
}


check_color() {
  if test -t 1; then
    ncolors=$(tput colors)
    if test -n "$ncolors" && test $ncolors -ge 8; then
      bold="$(tput bold)"
      underline="$(tput smul)"
      standout="$(tput smso)"
      normal="$(tput sgr0)"
      black="$(tput setaf 0)"
      red="$(tput setaf 1)"
      green="$(tput setaf 2)"
      yellow="$(tput setaf 3)"
      blue="$(tput setaf 4)"
      magenta="$(tput setaf 5)"
      cyan="$(tput setaf 6)"
      white="$(tput setaf 7)"
      blink="$(tput blink)"
    fi
  fi
}

pr_info() {
  echo -e "${green}$1${normal}"
}

pr_warm() {
  echo -e "${yellow}$1${normal}"
}

pr_error() {
  echo -e "${bold}${red}$1${normal}"
}

die() { pr_error "$*" 1>&2 ; exit 1; }

while [[ $1 = -?* ]]; do
  case $1 in
    -h|--help) display_usage >&2;;
    -u|--username) shift;login=" ${login} -u $1";;
    -p|--password) shift;login=" ${login} -p $1";;
    -H|--url) shift;url=$1;;
    -n|--nocolors) colors=0;;
    *) die "invalid option: '$1'." ;;
  esac
  shift
done

if [[ ${colors} -eq 1 ]]; then
  check_color
fi
if [[ -z "${login}" &&  $(id -u) -ne 0 ]]; then
  echo "This script must be run as root for local"
  exit 1
fi

ReturnCode=0
output=$(runilorest "login ${url}")
if [ $? -ne 0 ]; then
  echo ${output}
  exit 1
fi

pr_info "*** Reminder: This script queries the HPE Smart Array controller through the iLO and thus will only detect drives attached to HPE Smart Array controllers. Drives in external storage enclosures where the enclosures have their own storage controller that manage their installed drives will not be detected. For these situations, further analysis will be required to confirm whether affected drives are installed. Review the iLO GUI ‘System Information -> Storage' tab ---> only drives listed on that page will be evaluated by this script. ***"

GetServerInfo
pr_info "The following discovered drives need a firmware update to Version HPD7 before PowerOnHours exceed 32,768 hours:"
output=$(runilorest "list --selector HpeSmartStorageDiskDrive get model Firmwareversion location poweronhours")
if [ $? -ne 0 ]; then
  echo ${output}
  exit 1
fi
for CMD in $(echo -e "${output}\n"|sed 's/^\s*$/+++/g'); do
  key=$(echo ${CMD}|cut -d"=" -f1)
  value=$(echo ${CMD}|cut -d"=" -f2)
  if [[ "${key}" == "VersionString" ]]; then
    Versionstring=$value
  fi
  if [[ "${key}" == "Location" ]]; then
    Location=$value
  fi
  if [[ "${key}" == "Model" ]]; then
    Model=$value
  fi
  if [[ "${key}" == "PowerOnHours" ]]; then
    if [[ "${value}" == "None" ]]; then
      PowerOnHours="0"
    else
      PowerOnHours=${value}
    fi
  fi
  if [[ "${CMD}" == "+++" ]]; then
    if [[ "$(check_model ${Model})" -eq "1" ]]; then
      if [[ "${Versionstring}" != "HPD7" ]]; then
	ReturnCode=1
        if [[  "${PowerOnHours}" -gt "32000" ]]; then
          pr_warm "Failure ${blink}${bold}${red}IMMINENT${normal} ${Model} ${Versionstring} ${yellow}Location${normal} ${Location} ${yellow}Powen on${normal} ${PowerOnHours} hours"
        else
          pr_warm "Failure ${blink}${bold}${red}APPROACHING ${normal} ${Model} ${Versionstring} ${yellow}Location${normal} ${Location} ${yellow}Power on${normal} ${PowerOnHours} hours"
        fi
      fi
    fi
  fi
done
if [[  "${ReturnCode}" -eq 0 ]]; then
  pr_info "No affected hard drives found"
fi
IlorestLogout
exit ${ReturnCode}
