param ($url, $username, $password, $csv)

$ErrorActionPreference = "Stop"
$date = Get-Date
$ProblemModels = "EK0800JVYPN", "EO1600JVYPP", "MK0800JVYPQ", "MO1600JVYPR"

$global:ReturnCode=0
$global:highsecurity=""
$global:systemoutput=$null

function AddProperty($toExtend){
    if($global:systemoutput.GetType().Name -eq "PSCustomObject"){
        foreach($Property in $global:systemoutput | Get-Member -type NoteProperty, Property){
            if($toExtend.$($Property.Name) -eq $null){
              $toExtend | Add-Member -MemberType NoteProperty -Value $global:systemoutput.$($Property.Name) -Name $Property.Name `
            }
        }
    }
    return $toExtend
}
function IlorestLogout{
    HighSecurityRun("logout")  | Out-Null
    if ($url){
        Remove-Item -Path ${url}
    }
}

function HighSecurityRun{
    param($cl)
    if ($url){
        $torun = ".\ilorest.exe --cache-dir=$url --nologo $cl" + $global:highsecurity
    } else {
        $torun = ".\ilorest.exe --nologo $cl" + $global:highsecurity
    }
    Invoke-Expression $torun
}

function GetSmartArray{
    param($hdinfo, $arrays, $data)
    foreach($array in $arrays){
        Try{
            $arraypath=$array.'@odata.id'
            $hdpath=$hdinfo.'@odata.id'
        } Catch {
            $hdpath=$hdinfo.links.self.href
            $arraypath=$array.links.self.href
        }
        if ($hdpath.Contains($arraypath)){
            $data | Add-Member -Name "CurrentOperatingMode" -Value $array.CurrentOperatingMode -MemberType NoteProperty
            $data | Add-Member -Name "SmartArrayModel" -Value $array.Model -MemberType NoteProperty
            break
        }
    }

    return $hdinfo
}

function SearchForProblems {
    $hddata = ConvertFrom-Json -InputObject (HighSecurityRun("list -j")| Out-String)
    $arraydata = ConvertFrom-Json -InputObject (HighSecurityRun("list --select HpeSmartStorageArrayController. -j")| Out-String)
    
    foreach($model in $ProblemModels){
        foreach($harddrive in $hddata){
            if ($harddrive.Model -eq $model){
                $harddrive.FirmwareVersion = $harddrive.FirmwareVersion.Current.VersionString
                if ($harddrive.FirmwareVersion -ne "HPD7"){
                    $global:ReturnCode=1
                    $returndata = [PSCustomObject]@{}
                    $returndata | Add-Member -Name "Location" -Value $harddrive.Location -MemberType NoteProperty
                    $returndata | Add-Member -Name "Model" -Value $harddrive.Model -MemberType NoteProperty
                    $returndata | Add-Member -Name "PowerOnHours" -Value $harddrive.PowerOnHours -MemberType NoteProperty
                    $returndata | Add-Member -Name "SerialNumber" -Value $harddrive.SerialNumber -MemberType NoteProperty
                    $returndata | Add-Member -Name "FirmwareVersion" -Value $harddrive.FirmwareVersion -MemberType NoteProperty
                    if ($harddrive.PowerOnHours -gt 32000){
                        $returndata | Add-Member -Name "Failure" -Value "IMMINENT" -MemberType NoteProperty
                    } Elseif ($null -eq $harddrive.PowerOnHours){
                        $returndata | Add-Member -Name "Failure" -Value "IMMINENT" -MemberType NoteProperty
                    }
                    else {
                        $returndata | Add-Member -Name "Failure" -Value "APPROACHING" -MemberType NoteProperty
                    }
                    $harddrive = GetSmartArray $harddrive $arraydata $returndata
                    Write-Output "=================================================================="
                    Write-Output $returndata | Format-List
                    if ($csv){
                        $returndata = AddProperty $returndata
                        $returndata | Select-Object "iLO HostName", "iLO URL", "ServerModel", "SerialNumber", "ServerName", "Failure", "Location","Model","PowerOnHours", "FirmwareVersion","CurrentOperatingMode","SmartArrayModel" | Export-Csv -Path $csv -Append -NoTypeInformation
                    }
                }
            }
        }
    }
    if ($global:ReturnCode -eq 0){
        Write-Output "No affected hard drives found"
    }
}

function GetServerInfo {
    Write-Output "Server Info:"
    Write-Output "=================================================================="

    $serverdata = ConvertFrom-Json -InputObject (HighSecurityRun("get ServerName SerialNumber --selector Bios. -j") | Out-String)
    if ($null -eq $serverdata){
        
        $serverdata = ConvertFrom-Json -InputObject '{"SerialNumber": "Unable to gather serial number from Bios"}'
        $serverdata | Add-Member -Name "ServerName" -Value "Unable to gather Server Name from Bios" -MemberType NoteProperty
    }

    $hostname = ConvertFrom-Json -InputObject (HighSecurityRun("get --selector ServiceRoot. -j") | Out-String)
    Try{
        $serverdata | Add-Member -Name "iLO HostName" -Value $hostname.Oem.Hpe.Manager[0].HostName -MemberType NoteProperty -ErrorAction SilentlyContinue
    } Catch {
        $serverdata | Add-Member -Name "iLO HostName" -Value $hostname.Oem.Hp.Manager[0].HostName -MemberType NoteProperty
    }

    $model = ConvertFrom-Json -InputObject (HighSecurityRun("get Model --selector ComputerSystem. -j") | Out-String)
    if ($null -eq $model){
        $serverdata = ConvertFrom-Json -InputObject '{"ServerModel": "Unable to gather Model"}'
    } else {
        $serverdata | Add-Member -Name "ServerModel" -Value $model.Model -MemberType NoteProperty
    }

    $serverdata | Add-Member -Name "iLO URL" -Value $url -MemberType NoteProperty

    Write-Output $serverdata | Format-List
    $global:systemoutput = $serverdata
}

function CheckErrorCode {
    $Failed = 0
    if ($LastExitCode -eq 23) {
        Write-Output "Unable to find Smart Storage Drive information."
        IlorestLogout
        $Failed=1
    } Elseif ($LastExitCode -eq 34){
        Write-Output "If running locally CHIF driver is required. If running remotely, -url, -username, and -password arguments are required."
        $Failed=1
    }
    Elseif ($LastExitCode -eq 32){
        $Failed=1
    }
    Elseif ($LastExitCode -eq 75){
        Write-Output "Local Higher Security Mode is detected, please include -username and -password with iLO credentials."
        $Failed=1
    }
    Elseif ($LastExitCode -eq 5){
        Write-Output "If running locally you must run this script as an Administrator."
        $Failed=1
    }
    Elseif ($LastExitCode -ne 0){
        Write-Output "A general error occurred."
        IlorestLogout
        $Failed=1
    }

    if ($Failed -eq 1){
        $url | Out-File -FilePath ".\failed_systems.log" -Append
        exit -1
    }
}

Write-Output "SSD Firmware Check $date"
Write-Output "=================================================================="

Write-Output "*** Reminder: This script queries the HPE Smart Array controller through the iLO and thus will only detect drives attached to HPE Smart Array controllers. Drives in external storage enclosures where the enclosures have their own storage controller that manage their installed drives will not be detected. For these situations, further analysis will be required to confirm whether affected drives are installed. Review the iLO GUI ‘System Information -> Storage' tab ---> only drives listed on that page will be evaluated by this script. ***"

if ($url -and $username -and $password){
    #Remote login
    .\ilorest.exe --nologo --cache-dir=$url login $url -u $username -p $password | Out-Null
} Elseif ($username -and $password) {
    #Local High Security login
    $global:highsecurity = " -u " + $username + " -p " + $password
    .\ilorest.exe --nologo login -u $username -p $password | Out-Null
} else {
    #Local login
    .\ilorest.exe --nologo login | Out-Null
}
CheckErrorCode
GetServerInfo
HighSecurityRun("select HpeSmartStorageDiskDrive.")
CheckErrorCode
<#Write-Output "Current Drives Found:"
HighSecurityRun("get Location Model")
CheckErrorCode#>
Write-Output "The following discovered drives need a firmware update to Version HPD7 before PowerOnHours exceed 32,768 hours:"
SearchForProblems
IlorestLogout

exit $global:ReturnCode